def resolve(module, args=None, kwargs=None):
    from . import modules
    if args is None:
        args = ()
    if kwargs is None:
        kwargs = {}
    try:
        handler = modules.get(module)
    except module.ModuleDoesNotExist:
        return ''
    return handler(args, kwargs)



from django.template.defaulttags import kwarg_re
from django import template
from django.conf import settings

register = template.Library()


def parse_args(parser, bits):
    # see django.templatetags.future: url, line 80
    args = []
    kwargs = {}
    asvar = None
    if len(bits) >= 2 and bits[-2] == 'as':
        asvar = bits[-1]
        bits = bits[:-2]

    if len(bits):
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise template.TemplateSyntaxError("Malformed arguments to iurl tag")
            name, value = match.groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
            else:
                args.append(parser.compile_filter(value))
    
    return args, kwargs, asvar


class InternalUrlNode(template.Node):
    def __init__(self, module_name, args, kwargs, asvar):
        self.module_name = module_name
        self.args, self.kwargs, self.asvar = args, kwargs, asvar
    
    def render(self, context):
        from .. import modules
        module_name = self.module_name.resolve(context)
        args = [arg.resolve(context) for arg in self.args]
        kwargs = dict([(smart_str(k, 'ascii'), v.resolve(context))
                       for k, v in self.kwargs.iteritems()])
        try:
            module = modules.get(module_name)
        except modules.ModuleDoesNotExist:
            raise RuntimeError("valid module required")
        url = module(args, kwargs)
        if self.asvar:
            context[self.asvar] = url
            return ''
        return url


@register.tag
def iurl(parser, token):
    """Display a form and handles form-saving
    
    {% iurl "module" arg1 arg2 kwarg1=foo kwarg2=bar %}
    """
    parts = token.split_contents()
    if len(parts) < 2:
        raise template.TemplateSyntaxError("%r tag requires as least one argument" % parts[0])
    module_name = parser.compile_filter(parts[1])
    args, kwargs, asvar = parse_args(parser, parts[2:])
    return InternalUrlNode(module_name, args, kwargs, asvar)




class ModuleDoesNotExist(KeyError):
	pass

_modules = {}

def register(name, module):
	_modules[name] = module

def get(name):
	try:
		return _modules[name]
	except KeyError:
		raise ModuleDoesNotExist()

def get_all():
	return _modules




from django.conf import settings
from django.utils.importlib import import_module
from django.utils.module_loading import module_has_submodule

for app in settings.INSTALLED_APPS:
    if '.' not in app:
        continue
    mod = import_module(app)
    try:
        import_module('%s.internal_urls' % app)
    except:
        if module_has_submodule(mod, 'internal_urls'):
            raise




from . import register

def django_url(args, kwargs):
    from django.core.urlresolvers import reverse, NoReverseMatch
    if len(args) < 1:
        return ''
    url = args[0]
    args = args[1:]
    try:
        return reverse(url, args=args, kwargs=kwargs)
    except NoReverseMatch:
        return ''

register('url', django_url)




from setuptools import setup, find_packages

setup(
    name = "django_internal_urls",
    version = "0.1.0-2",
    description = 'Add modular url callbacks',
    author = 'David Danier',
    author_email = 'david.danier@team23.de',
    url = 'https://github.com/ddanier/django_internal_urls',
    #long_description=open('README.rst', 'r').read(),
    packages = [
        'django_internal_urls',
        'django_internal_urls.templatetags',
    ],
    install_requires = [
        'Django >=1.3',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)

